package com.nisum.bookstore.service.impl;

import com.nisum.bookstore.dto.BookDto;
import com.nisum.bookstore.entity.Book;
import com.nisum.bookstore.repo.BookRepo;
import com.nisum.bookstore.service.BookService;
import com.nisum.bookstore.utils.AppUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class BookServiceImpl implements BookService {

    @Autowired
    BookRepo bookRepo;

    @Override
    public Flux<BookDto> getBooks() {
        return bookRepo.findAll().map( book -> AppUtils.entityToDto(book) );
    }

    @Override
    public Mono<BookDto> getBookById(String id) {
        return bookRepo.findById(id).map( book -> AppUtils.entityToDto(book) ).switchIfEmpty(Mono.empty());
    }

    @Override
    public Mono<BookDto> addBook(Mono<BookDto> book) {
        return book.map( b -> AppUtils.dtoToEntity(b) ).flatMap(bookRepo::save).map( b -> AppUtils.entityToDto(b) );
    }

    @Override
    public Mono<BookDto> updateBook(String id, Mono<BookDto> book) {
        return bookRepo.findById( id )
                .flatMap( book1 -> book.map(AppUtils::dtoToEntity))
                .doOnNext(e -> e.setId(id))
                .flatMap(bookRepo::save)
                .map( o -> AppUtils.entityToDto(o));
    }

    @Override
    public Mono<Void> deleteBook(String id) {
        return bookRepo.deleteById(id);
    }

}
